@@ -24,7 +24,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtktextdisplay.h>
@@ -1228,10 +1227,8 @@ gnome_canvas_rich_text_button_press_event(GnomeCanvasItem *item,
 		return TRUE;
 	}
 	else if (event->button == 2 && event_type == GDK_BUTTON_PRESS) {
-		gtk_text_buffer_paste_clipboard(
-			get_buffer(text),
-			gtk_clipboard_get (GDK_SELECTION_PRIMARY),
-			&iter, text->_priv->editable);
+		gtk_text_buffer_paste_primary(
+			get_buffer(text), &iter, text->_priv->editable);
 	}
 		
 	return FALSE;
@@ -1420,9 +1417,7 @@ gnome_canvas_rich_text_cut_clipboard(GnomeCanvasRichText *text)
 	g_return_if_fail(text);
 	g_return_if_fail(get_buffer(text));
 
-	gtk_text_buffer_cut_clipboard(get_buffer(text),
-				      gtk_clipboard_get (GDK_SELECTION_PRIMARY),
-				      text->_priv->editable);
+	gtk_text_buffer_cut_clipboard(get_buffer(text), text->_priv->editable);
 } /* gnome_canvas_rich_text_cut_clipboard */
 
 void
@@ -1431,8 +1426,7 @@ gnome_canvas_rich_text_copy_clipboard(GnomeCanvasRichText *text)
 	g_return_if_fail(text);
 	g_return_if_fail(get_buffer(text));
 
-	gtk_text_buffer_copy_clipboard(get_buffer(text),
-				       gtk_clipboard_get (GDK_SELECTION_PRIMARY));
+	gtk_text_buffer_copy_clipboard(get_buffer(text));
 } /* gnome_canvas_rich_text_cut_clipboard */
 
 void
@@ -1441,10 +1435,7 @@ gnome_canvas_rich_text_paste_clipboard(GnomeCanvasRichText *text)
 	g_return_if_fail(text);
 	g_return_if_fail(get_buffer(text));
 
-	gtk_text_buffer_paste_clipboard(get_buffer(text),
-					gtk_clipboard_get (GDK_SELECTION_PRIMARY),
-					NULL,
-					text->_priv->editable);
+	gtk_text_buffer_paste_clipboard(get_buffer(text), text->_priv->editable);
 } /* gnome_canvas_rich_text_cut_clipboard */
 
 static gint
@@ -1660,11 +1651,11 @@ gnome_canvas_rich_text_set_attributes_from_style(GnomeCanvasRichText *text,
 	values->appearance.bg_color = style->base[GTK_STATE_NORMAL];
 	values->appearance.fg_color = style->fg[GTK_STATE_NORMAL];
 	
-	if (values->font)
-		pango_font_description_free (values->font);
-
-	values->font = pango_font_description_copy (style->font_desc);
-
+	if (values->font.family_name)
+		g_free (values->font.family_name);
+	
+	values->font = *style->font_desc;
+	values->font.family_name = g_strdup (style->font_desc->family_name);
 } /* gnome_canvas_rich_text_set_attributes_from_style */
 
 static void
@@ -1777,8 +1768,6 @@ adjust_for_anchors(GnomeCanvasRichText *text, double *ax, double *ay)
 	case GTK_ANCHOR_SE:
 		x -= text->_priv->width;
 		break;
-	default:
-		break;
 	}
 
 	/* Y coordinates */
@@ -1799,8 +1788,6 @@ adjust_for_anchors(GnomeCanvasRichText *text, double *ax, double *ay)
 	case GTK_ANCHOR_SE:
 		y -= text->_priv->height;
 		break;
-	default:
-		break;
 	}
 
 	if (ax)
@@ -1926,16 +1913,13 @@ gnome_canvas_rich_text_draw(GnomeCanvasItem *item, GdkDrawable *drawable,
 	y2 = c2.y;
 
 	gtk_text_layout_set_screen_width(text->_priv->layout, x2 - x1);
-      
-        /* FIXME: should last arg be NULL? */
+
 	gtk_text_layout_draw(
 		text->_priv->layout,
 		GTK_WIDGET(item->canvas),
 		drawable,
-		GTK_WIDGET (item->canvas)->style->text_gc[GTK_STATE_NORMAL],
 		x - x1, y - y1,
-		0, 0, (x2 - x1) - (x - x1), (y2 - y1) - (y - y1),
-		NULL);
+		0, 0, (x2 - x1) - (x - x1), (y2 - y1) - (y - y1));
 } /* gnome_canvas_rich_text_draw */
 
 static void
@@ -1944,7 +1928,6 @@ gnome_canvas_rich_text_render(GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	g_warning ("rich text item not implemented for anti-aliased canvas");
 } /* gnome_canvas_rich_text_render */
 
-#if 0
 static GtkTextTag *
 gnome_canvas_rich_text_add_tag(GnomeCanvasRichText *text, char *tag_name,
 			       int start_offset, int end_offset, 
@@ -1980,4 +1963,3 @@ gnome_canvas_rich_text_add_tag(GnomeCanvasRichText *text, char *tag_name,
 
 	return tag;
 } /* gnome_canvas_rich_text_add_tag */
-#endif
