@@ -9,7 +9,7 @@
  *
  * Authors: Federico Mena <federico@nuclecu.unam.mx>
  *          Raph Levien <raph@acm.org>
- *          Lauris Kaplinski <lauris@ximian.com>
+ *          Lauris Kaplinski <lauris@ariman.ee>
  *          Miguel de Icaza <miguel@kernel.org>
  *          Cody Russell <bratsche@gnome.org>
  *          Rusty Conover <rconover@bangtail.net>
@@ -35,12 +35,10 @@
 #include <libart_lgpl/art_bpath.h>
 #include <libart_lgpl/art_vpath_bpath.h>
 #include <libart_lgpl/art_svp.h>
-#include <libart_lgpl/art_svp_point.h>
 #include <libart_lgpl/art_svp_vpath.h>
 #include <libart_lgpl/art_vpath_dash.h>
 #include <libart_lgpl/art_svp_wind.h>
-#include <libart_lgpl/art_svp_intersect.h>
-#include <libart_lgpl/art_rect_svp.h>
+#include <libart_lgpl/art_svp_point.h>
 
 enum {
 	PROP_0,
@@ -81,8 +79,7 @@ static void   gnome_canvas_shape_draw        (GnomeCanvasItem *item, GdkDrawable
 static double gnome_canvas_shape_point       (GnomeCanvasItem *item, double x, double y,
                                               int cx, int cy, GnomeCanvasItem **actual_item);
 static void   gnome_canvas_shape_render      (GnomeCanvasItem *item, GnomeCanvasBuf *buf);
-static void   gnome_canvas_shape_bounds      (GnomeCanvasItem *item,
-					      double *x1, double *y1, double *x2, double *y2);
+
 
 static gulong get_pixel_from_rgba (GnomeCanvasItem *item, guint32 rgba_color);
 static guint32 get_rgba_from_color (GdkColor * color);
@@ -225,7 +222,6 @@ gnome_canvas_shape_class_init (GnomeCanvasShapeClass *class)
 	item_class->draw = gnome_canvas_shape_draw;
 	item_class->point = gnome_canvas_shape_point;
 	item_class->render = gnome_canvas_shape_render;
-	item_class->bounds = gnome_canvas_shape_bounds;
 }
 
 static void
@@ -247,7 +243,7 @@ gnome_canvas_shape_init (GnomeCanvasShape *shape)
 	shape->priv->cap = GDK_CAP_BUTT;
 	shape->priv->join = GDK_JOIN_MITER;
 	shape->priv->wind = ART_WIND_RULE_ODDEVEN;
-	shape->priv->miterlimit = 10.43;	   /* X11 default */
+	shape->priv->miterlimit = 11.0;			/* PS default? */
 
 	shape->priv->dash.n_dash = 0;
 	shape->priv->dash.dash = NULL;
@@ -289,33 +285,37 @@ gnome_canvas_shape_destroy (GtkObject *object)
 
 /**
  * gnome_canvas_shape_set_path_def:
- * @shape: a GnomeCanvasShape
+ * @object: a GnomeCanvasShape
  * @def: a GnomeCanvasPathDef 
  *
  * This function sets the the GnomeCanvasPathDef used by the
- * GnomeCanvasShape. Notice, that it does not request updates, as
- * it is meant to be used from item implementations, from inside
- * update queue.
+ * GnomeCanvasShape.
  */
  
 void
-gnome_canvas_shape_set_path_def (GnomeCanvasShape *shape, GnomeCanvasPathDef *def) 
+gnome_canvas_shape_set_path_def (GObject *object,
+				 GnomeCanvasPathDef *def) 
 {
+	GnomeCanvasItem *item;
+	GnomeCanvasShape *shape;
 	GnomeCanvasShapePriv *priv;
+	
+	g_return_if_fail(GNOME_IS_CANVAS_SHAPE(object));
 
-	g_return_if_fail (shape != NULL);
-	g_return_if_fail (GNOME_IS_CANVAS_SHAPE (shape));
-
+	shape = GNOME_CANVAS_SHAPE(object);
 	priv = shape->priv;
 
-	if (priv->path) {
-		gnome_canvas_path_def_unref (priv->path);
+	item = GNOME_CANVAS_ITEM(object);
+
+	if(priv->path)
+		gnome_canvas_path_def_unref(priv->path);
+
+	if(def) {
+		priv->path = gnome_canvas_path_def_duplicate(def);
+	} else {
 		priv->path = NULL;
 	}
-
-	if (def) {
-		priv->path = gnome_canvas_path_def_duplicate (def);
-	}
+	
 }
 
 static void
@@ -349,29 +349,19 @@ gnome_canvas_shape_set_property (GObject      *object,
 			priv->fill_set = TRUE;
 			priv->fill_rgba = get_rgba_from_color (&color);
 			if (gdk) gdk->fill_pixel = color.pixel;
-		} else if (priv->fill_set)
+		} else {
 			priv->fill_set = FALSE;
-		else
-			break;
+		}
 
 		gnome_canvas_item_request_update (item);
 		break;
 
 	case PROP_FILL_COLOR_GDK:
 		colorptr = g_value_get_boxed (value);
-		if (colorptr != NULL) {
-			priv->fill_set = TRUE;
-			priv->fill_rgba = get_rgba_from_color (colorptr);
-			if (gdk) {
-				GdkColormap *colormap = gtk_widget_get_colormap (GTK_WIDGET (item->canvas));
-				GdkColor tmp = *colorptr;
-				gdk_rgb_find_color (colormap, &tmp);
-				gdk->fill_pixel = tmp.pixel;
-			}
-		} else if (priv->fill_set)
-			priv->fill_set = FALSE;
-		else
-			break;
+
+		priv->fill_set = TRUE;
+		priv->fill_rgba = get_rgba_from_color (colorptr);
+		if (gdk) gdk->fill_pixel = colorptr->pixel;
 
 		gnome_canvas_item_request_update (item);
 		break;
@@ -389,29 +379,19 @@ gnome_canvas_shape_set_property (GObject      *object,
 			priv->outline_set = TRUE;
 			priv->outline_rgba = get_rgba_from_color (&color);
 			if (gdk) gdk->outline_pixel = color.pixel;
-		} else if (priv->outline_set)
+		} else {
 			priv->outline_set = FALSE;
-		else
-			break;
+		}
 
 		gnome_canvas_item_request_update (item);
 		break;
 
 	case PROP_OUTLINE_COLOR_GDK:
 		colorptr = g_value_get_boxed (value);
-		if (colorptr != NULL) {
-			priv->outline_set = TRUE;
-			priv->outline_rgba = get_rgba_from_color (colorptr);
-			if (gdk) {
-				GdkColormap *colormap = gtk_widget_get_colormap (GTK_WIDGET (item->canvas));
-				GdkColor tmp = *colorptr;
-				gdk_rgb_find_color (colormap, &tmp);
-				gdk->outline_pixel = tmp.pixel;
-			}
-		} else if (priv->outline_set)
-			priv->outline_set = FALSE;
-		else
-			break;
+
+		priv->outline_set = TRUE;
+		priv->outline_rgba = get_rgba_from_color (colorptr);
+		if (gdk) gdk->outline_pixel = colorptr->pixel;
 
 		gnome_canvas_item_request_update (item);
 		break;
@@ -426,14 +406,14 @@ gnome_canvas_shape_set_property (GObject      *object,
 
 	case PROP_FILL_STIPPLE:
 		if (gdk) {
-			set_stipple (gdk->fill_gc, &gdk->fill_stipple, (GdkBitmap*) g_value_get_object (value), FALSE);
+			set_stipple (gdk->fill_gc, &gdk->fill_stipple, (GdkBitmap *)g_value_get_object (value), FALSE);
 			gnome_canvas_item_request_update (item);
 		}
 		break;
 
 	case PROP_OUTLINE_STIPPLE:
 		if (gdk) {
-			set_stipple (gdk->outline_gc, &gdk->outline_stipple, (GdkBitmap*) g_value_get_object (value), FALSE);
+			set_stipple (gdk->outline_gc, &gdk->outline_stipple, (GdkBitmap *)g_value_get_object (value), FALSE);
 			gnome_canvas_item_request_update (item);
 		}
 		break;
@@ -453,7 +433,7 @@ gnome_canvas_shape_set_property (GObject      *object,
 		break;
 
 	case PROP_WIND:
-		priv->wind = g_value_get_uint (value);
+		priv->wind = g_value_get_enum (value);
 		gnome_canvas_item_request_update (item);
 		break;
 
@@ -488,7 +468,6 @@ gnome_canvas_shape_set_property (GObject      *object,
 		gnome_canvas_item_request_update (item);
 		break;
 	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
 	}
 }
@@ -501,17 +480,20 @@ gnome_canvas_shape_set_property (GObject      *object,
 static void
 get_color_value (GnomeCanvasShape *shape, gulong pixel, GValue *value)
 {
-	GnomeCanvas *canvas = GNOME_CANVAS_ITEM (shape)->canvas;
-	GdkColormap *colormap = gtk_widget_get_colormap (GTK_WIDGET (canvas));
-	GdkColor color;
+	GdkColor *color;
+	GdkColormap *colormap;
 
-	gdk_colormap_query_color (colormap, pixel, &color);
-	g_value_set_boxed (value, &color);
+	color = g_new (GdkColor, 1);
+	color->pixel = pixel;
+
+	colormap = gtk_widget_get_colormap (GTK_WIDGET (shape));
+	gdk_rgb_find_color (colormap, color);
+	g_value_set_boxed (value, color);
 }
 
 /**
  * gnome_canvas_shape_get_path_def:
- * @shape: a GnomeCanvasShape
+ * @object: a GnomeCanvasShape
  *
  * This function returns the #GnomeCanvasPathDef that the shape
  * currently uses.  It adds a reference to the #GnomeCanvasPathDef and
@@ -522,13 +504,11 @@ get_color_value (GnomeCanvasShape *shape, gulong pixel, GValue *value)
  */
  
 GnomeCanvasPathDef *
-gnome_canvas_shape_get_path_def (GnomeCanvasShape *shape)
-{
+gnome_canvas_shape_get_path_def(GObject *object) {
+	GnomeCanvasShape *shape;
 	GnomeCanvasShapePriv *priv;
 	
-	g_return_val_if_fail (shape != NULL, NULL);
-	g_return_val_if_fail (GNOME_IS_CANVAS_SHAPE (shape), NULL);
-
+	shape = GNOME_CANVAS_SHAPE(object);
 	priv = shape->priv;
 
 	if (priv->path) {
@@ -545,11 +525,15 @@ gnome_canvas_shape_get_property (GObject     *object,
                                  GValue      *value,
                                  GParamSpec  *pspec)
 {
-	GnomeCanvasItem         *item = GNOME_CANVAS_ITEM (object);
-	GnomeCanvasShape        *shape = GNOME_CANVAS_SHAPE (object);
-	GnomeCanvasShapePriv    *priv = shape->priv;
+	GnomeCanvasItem         *item;
+	GnomeCanvasShape        *shape;
+	GnomeCanvasShapePriv    *priv;
 	GnomeCanvasShapePrivGdk *gdk;
 
+	item = (GnomeCanvasItem *) object;
+	shape = (GnomeCanvasShape *) object;
+	priv = shape->priv;
+
 	if (!item->canvas->aa) {
 		gcbp_ensure_gdk (shape);
 		gdk = priv->gdk;
@@ -580,27 +564,27 @@ gnome_canvas_shape_get_property (GObject     *object,
 		break;
 
 	case PROP_OUTLINE_COLOR_RGBA:
-		g_value_set_uint (value, priv->outline_rgba);
+		g_value_set_uint (value, priv->fill_rgba);
 		break;
 
 	case PROP_FILL_STIPPLE:
 		if (gdk) {
-			g_value_set_object (value, gdk->fill_stipple);
+			g_value_set_boxed (value, gdk->fill_stipple);
 		} else {
-			g_value_set_object (value, NULL);
+			g_value_set_boxed (value, NULL);
 		}
 		break;
 
 	case PROP_OUTLINE_STIPPLE:
 		if (gdk) {
-			g_value_set_object (value, gdk->outline_stipple);
+			g_value_set_boxed (value, gdk->outline_stipple);
 		} else {
-			g_value_set_object (value, NULL);
+			g_value_set_boxed (value, NULL);
 		}
 		break;
 
 	case PROP_WIND:
-		g_value_set_uint (value, priv->wind);
+		g_value_set_enum (value, priv->wind);
 		break;
 
 	case PROP_CAP_STYLE:
@@ -611,18 +595,6 @@ gnome_canvas_shape_get_property (GObject     *object,
 		g_value_set_enum (value, priv->join);
 		break;
 
-	case PROP_WIDTH_PIXELS:
-		g_value_set_uint (value, priv->width);
-		break;
-
-	case PROP_MITERLIMIT:
-		g_value_set_double (value, priv->miterlimit);
-		break;
-
-	case PROP_DASH:
-		g_value_set_pointer (value, priv->dash.dash);
-		break;
-		
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -789,10 +761,10 @@ gnome_canvas_shape_draw (GnomeCanvasItem *item,
 			len = GPOINTER_TO_INT (l->data);
 
 			gdk_draw_polygon (drawable,
-					  gdk->outline_gc,
-					  FALSE,
-					  &dpoints[pos],
-					  len);
+				gdk->outline_gc,
+				FALSE,
+				&dpoints[pos],
+				len);
 
 			pos += len;
 		}
@@ -826,33 +798,21 @@ gnome_canvas_shape_update_gdk (GnomeCanvasShape * shape, double * affine, ArtSVP
 {
 	GnomeCanvasShapePriv * priv;
 	GnomeCanvasShapePrivGdk * gdk;
-	int x1 = 0, y1 = 0, x2 = 0, y2 = 0;
-	gboolean bbox_set = FALSE;
-	gint width = 0; /* silence gcc */
-	
+
 	g_assert (!((GnomeCanvasItem *) shape)->canvas->aa);
 
 	priv = shape->priv;
 	gdk = priv->gdk;
 	g_assert (gdk != NULL);
 
-	if (priv->outline_set) {
+	if (shape->priv->outline_set) {
 		GdkLineStyle style;
+		gint width;
 
 		if (priv->width_pixels) {
-			width = (int) floor (priv->width + 0.5);
-			/* Never select 0 pixels unless the user asked for it,
-			 * since that is the X11 zero width lines are non-specified */
-			if (priv->width_pixels != 0 && width == 0) {
-				width = 1;
-			}
+			width = (int) priv->width;
 		} else {
-			width = (int) floor ((priv->width * priv->scale) + 0.5);
-			/* Never select 0 pixels unless the user asked for it,
-			 * since that is the X11 zero width lines are non-speciifed */
-			if (priv->width != 0 && width == 0) {
-				width = 1;
-			}
+			width = (int) (priv->width * priv->scale);
 		}
 
 		/* If dashed, set it in GdkGC */
@@ -963,24 +923,13 @@ gnome_canvas_shape_update_gdk (GnomeCanvasShape * shape, double * affine, ArtSVP
 
 			path = (GnomeCanvasPathDef *) clist->data;
 			bpath = gnome_canvas_path_def_bpath (path);
-			vpath = art_bez_path_to_vec (bpath, 0.1);
+			vpath = art_bez_path_to_vec (bpath, 0.5);
 			for (len = 0; vpath[len].code != ART_END; len++) ;
 
 			gnome_canvas_shape_ensure_gdk_points (gdk, len);
 			for (i = 0; i < len; i++) {
-				gdk->points[pos + i].x = (gint) floor (vpath[i].x + 0.5);
-				gdk->points[pos + i].y = (gint) floor (vpath[i].y + 0.5);
-
-				if (bbox_set) {
-					x1 = MIN (x1, gdk->points[pos + i].x);
-					x2 = MAX (x2, gdk->points[pos + i].x);
-					y1 = MIN (y1, gdk->points[pos + i].y);
-					y2 = MAX (y2, gdk->points[pos + i].y);
-				} else {
-					bbox_set = TRUE;
-					x1 = x2 = gdk->points[pos + i].x;
-					y1 = y2 = gdk->points[pos + i].y;
-				}
+				gdk->points[pos + i].x = (gint16) vpath[i].x;
+				gdk->points[pos + i].y = (gint16) vpath[i].y;
 			}
 			gdk->num_points += len;
 
@@ -1005,24 +954,13 @@ gnome_canvas_shape_update_gdk (GnomeCanvasShape * shape, double * affine, ArtSVP
 
 			path = (GnomeCanvasPathDef *) olist->data;
 			bpath = gnome_canvas_path_def_bpath (path);
-			vpath = art_bez_path_to_vec (bpath, 0.1);
+			vpath = art_bez_path_to_vec (bpath, 0.5);
 			for (len = 0; vpath[len].code != ART_END; len++) ;
 
 			gnome_canvas_shape_ensure_gdk_points (gdk, len);
 			for (i = 0; i < len; i++) {
-				gdk->points[pos + i].x = (gint) floor (vpath[i].x + 0.5);
-				gdk->points[pos + i].y = (gint) floor (vpath[i].y + 0.5);
-				
-				if (bbox_set) {
-					x1 = MIN (x1, gdk->points[pos + i].x);
-					x2 = MAX (x2, gdk->points[pos + i].x);
-					y1 = MIN (y1, gdk->points[pos + i].y);
-					y2 = MAX (y2, gdk->points[pos + i].y);
-				} else {
-					bbox_set = TRUE;
-					x1 = x2 = gdk->points[pos + i].x;
-					y1 = y2 = gdk->points[pos + i].y;
-				}
+				gdk->points[pos + i].x = (gint16) vpath[i].x;
+				gdk->points[pos + i].y = (gint16) vpath[i].y;
 			}
 			gdk->num_points += len;
 
@@ -1039,20 +977,6 @@ gnome_canvas_shape_update_gdk (GnomeCanvasShape * shape, double * affine, ArtSVP
 
 	}
 
-	if (bbox_set) {
-		if (priv->outline_set && (priv->join == GDK_JOIN_MITER)) {
-			int stroke_border = ceil (10.43*width/2); /* 10.43 is the miter limit for X11 */
-			x1 -= stroke_border;
-			x2 += stroke_border;
-			y1 -= stroke_border;
-			y2 += stroke_border;
-		}
-		
-		gnome_canvas_update_bbox (GNOME_CANVAS_ITEM (shape),
-					  x1, y1,
-					  x2 + 1, y2 + 1);
-	}
-	
 }
 
 static void
@@ -1067,26 +991,25 @@ gnome_canvas_shape_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_p
 	priv = shape->priv;
 
 	/* Common part */
-	if (parent_class->update) {
+
+	if (parent_class->update)
 		(* parent_class->update) (item, affine, clip_path, flags);
-	}
 
 	/* Outline width */
+
 	shape->priv->scale = (fabs (affine[0]) + fabs (affine[3])) / 2.0;
 
 	/* Reset bbox */
-	if (item->canvas->aa) {
-		gnome_canvas_item_reset_bounds (item);
-	}
+
+	gnome_canvas_item_reset_bounds (item);
 	
 	/* Clipped fill SVP */
 
 	if ((shape->priv->fill_set) && (gnome_canvas_path_def_any_closed (shape->priv->path))) {
 		GnomeCanvasPathDef * cpath;
-		ArtSvpWriter *swr;
-		ArtVpath *vpath;
-		ArtBpath *abp;
-		ArtSVP *svp2;
+		ArtBpath * abp;
+		ArtVpath * vpath, * pvpath;
+		ArtSVP *tmp_svp;
 
 		/* Get closed part of path */
 
@@ -1096,38 +1019,44 @@ gnome_canvas_shape_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_p
 
 		/* Render, until SVP */
 
-		vpath = art_bez_path_to_vec (abp, 0.1);
+		vpath = art_bez_path_to_vec (abp, 0.25);
 		art_free (abp);
 
-		svp = art_svp_from_vpath (vpath);
+		pvpath = art_vpath_perturb (vpath);
 		art_free (vpath);
 
-		swr = art_svp_writer_rewind_new (shape->priv->wind);
-		art_svp_intersector (svp, swr);
-
-		svp2 = art_svp_writer_rewind_reap (swr);
+		svp = art_svp_from_vpath (pvpath);
+		art_free (pvpath);
+		
+		tmp_svp = art_svp_uncross (svp);
 		art_svp_free (svp);
 
+		svp = art_svp_rewind_uncrossed (tmp_svp, shape->priv->wind);
+		art_svp_free (tmp_svp);
+
 		if (item->canvas->aa) {
+
 			/* Update clipped path */
+
 			gnome_canvas_item_update_svp_clip (item,
-							   &shape->priv->fill_svp,
-							   svp2,
-							   clip_path);
+				&shape->priv->fill_svp,
+				svp,
+				clip_path);
 		} else {
-			if (priv->fill_svp) {
-				art_svp_free (priv->fill_svp);
-				priv->fill_svp = NULL;
-			}
+
 			/* No clipping */
-			shape->priv->fill_svp = svp2;
+
+			gnome_canvas_item_update_svp_clip (item,
+				&shape->priv->fill_svp,
+				svp,
+				NULL);
 		}
 	}
 
-	if (priv->outline_set && priv->path && !gnome_canvas_path_def_is_empty (priv->path)) {
+	if ((priv->outline_set) && (!gnome_canvas_path_def_is_empty (priv->path))) {
 		gdouble width;
 		ArtBpath * abp;
-		ArtVpath * vpath;
+		ArtVpath * vpath, * pvpath;
 
 		/* Set linewidth */
 
@@ -1143,44 +1072,48 @@ gnome_canvas_shape_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_p
 
 		abp = art_bpath_affine_transform (gnome_canvas_path_def_bpath (priv->path), affine);
 
-		vpath = art_bez_path_to_vec (abp, 0.1);
+		vpath = art_bez_path_to_vec (abp, 0.25);
 		art_free (abp);
 
+		pvpath = art_vpath_perturb (vpath);
+		art_free (vpath);
+
 		/* If dashed, apply dash */
 
 		if (priv->dash.dash != NULL)
 		{
-			ArtVpath *old = vpath;
+			ArtVpath *old = pvpath;
 			
-			vpath = art_vpath_dash (old, &priv->dash);
+			pvpath = art_vpath_dash (old, &priv->dash);
 			art_free (old);
 		}
 		
 		/* Stroke vpath to SVP */
 
-		svp = art_svp_vpath_stroke (vpath,
+		svp = art_svp_vpath_stroke (pvpath,
 					    gnome_canvas_join_gdk_to_art (priv->join),
 					    gnome_canvas_cap_gdk_to_art (priv->cap),
 					    width,
 					    priv->miterlimit,
 					    0.25);
-		art_free (vpath);
+		art_free (pvpath);
 
 		if (item->canvas->aa) {
+
 			/* Update clipped */
+
 			gnome_canvas_item_update_svp_clip (item, &priv->outline_svp, svp, clip_path);
+
 		} else {
-			if (priv->outline_svp) {
-				art_svp_free (priv->outline_svp);
-				priv->outline_svp = NULL;
-			}
+
 			/* No clipping (yet) */
-			shape->priv->outline_svp = svp;
+
+			gnome_canvas_item_update_svp_clip (item, &priv->outline_svp, svp, NULL);
 		}
 	}
 
 	/* Gdk requires additional handling */
-	
+
 	if (!item->canvas->aa) {
 		gnome_canvas_shape_update_gdk (shape, affine, clip_path, flags);
 	}
@@ -1194,29 +1127,30 @@ gnome_canvas_shape_point (GnomeCanvasItem *item, double x, double y,
 	double dist;
 	int wind;
 
-#if 0
-	/* fixme: This is just for debugging, canvas should ensure that */
-	/* fixme: IF YOU ARE SURE THAT IT IS CORRECT BEHAVIOUR, you can remove warning */
-	/* fixme: and make it to return silently */
-	g_return_val_if_fail (!item->canvas->need_update, 1e18);
-#endif
-
 	shape = GNOME_CANVAS_SHAPE (item);
 
 	/* todo: update? */
-	if (shape->priv->fill_set && shape->priv->fill_svp) {
-		wind = art_svp_point_wind (shape->priv->fill_svp, cx, cy);
-		if ((shape->priv->wind == ART_WIND_RULE_NONZERO) && (wind != 0)) {
-			*actual_item = item;
-			return 0.0;
-		}
-		if ((shape->priv->wind == ART_WIND_RULE_ODDEVEN) && ((wind & 0x1) != 0)) {
-			*actual_item = item;
-			return 0.0;
+	if (shape->priv->fill_set) {
+		if (!shape->priv->fill_svp) {
+			static gboolean need_warning = TRUE;
+			if (need_warning) {
+				need_warning = FALSE;
+				g_warning (G_STRLOC ": FIXME: shape->priv->fill_set == TRUE && shape->priv->fill_svp == NULL");
+			}
+		} else {
+			wind = art_svp_point_wind (shape->priv->fill_svp, cx, cy);
+			if ((shape->priv->wind == ART_WIND_RULE_NONZERO) && (wind != 0)) {
+				*actual_item = item;
+				return 0.0;
+			}
+			if ((shape->priv->wind == ART_WIND_RULE_ODDEVEN) && ((wind & 0x1) != 0)) {
+				*actual_item = item;
+				return 0.0;
+			}
 		}
 	}
 
-	if (shape->priv->outline_set && shape->priv->outline_svp) {
+	if (shape->priv->outline_set) {
 		wind = art_svp_point_wind (shape->priv->outline_svp, cx, cy);
 		if (wind) {
 			*actual_item = item;
@@ -1224,9 +1158,9 @@ gnome_canvas_shape_point (GnomeCanvasItem *item, double x, double y,
 		}
 	}
 
-	if (shape->priv->outline_set && shape->priv->outline_svp) {
+	if (shape->priv->outline_set) {
 		dist = art_svp_point_dist (shape->priv->outline_svp, cx, cy);
-	} else if (shape->priv->fill_set && shape->priv->outline_svp) {
+	} else if (shape->priv->fill_set) {
 		dist = art_svp_point_dist (shape->priv->fill_svp, cx, cy);
 	} else {
 		return 1e12;
@@ -1463,89 +1397,3 @@ gcbp_draw_ctx_unref (GCBPDrawCtx * ctx)
 	}
 }
 
-static void
-gnome_canvas_shape_bounds (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2)
-{
-	GnomeCanvasShape * shape;
-	GnomeCanvasShapePriv * priv;
-	ArtDRect bbox;
-	ArtSVP * svp;
-
-	shape = GNOME_CANVAS_SHAPE (item);
-
-	priv = shape->priv;
-
-	bbox.x0 = *x1;
-	bbox.y0 = *y1;
-	bbox.x1 = *x2;
-	bbox.y1 = *y2;
-
-	if (priv->outline_set && priv->path && !gnome_canvas_path_def_is_empty (priv->path)) {
-		gdouble width;
-		ArtVpath * vpath;
-
-		/* Set linewidth */
-
-		if (priv->width_pixels) {
-			width = priv->width;
-		} else {
-			width = priv->width * priv->scale;
-		}
-		
-		if (width < 0.5) width = 0.5;
-		
-		/* Render full path until vpath */
-
-		vpath = art_bez_path_to_vec (gnome_canvas_path_def_bpath (priv->path), 0.1);
-
-		/* If dashed, apply dash */
-
-		if (priv->dash.dash != NULL)
-		{
-			ArtVpath *old = vpath;
-			
-			vpath = art_vpath_dash (old, &priv->dash);
-			art_free (old);
-		}
-		
-		/* Stroke vpath to SVP */
-
-		svp = art_svp_vpath_stroke (vpath,
-					    gnome_canvas_join_gdk_to_art (priv->join),
-					    gnome_canvas_cap_gdk_to_art (priv->cap),
-					    priv->width,
-					    priv->miterlimit,
-					    0.25);
-		art_free (vpath);
-		art_drect_svp (&bbox, svp);
-		art_svp_free (svp);
-	} else if ((shape->priv->fill_set) && (gnome_canvas_path_def_any_closed (shape->priv->path))) {
-		GnomeCanvasPathDef *cpath;
-		ArtSvpWriter *swr;
-		ArtVpath *vpath;
-		ArtSVP *svp2;
-
-		/* Get closed part of path */
-		cpath = gnome_canvas_path_def_closed_parts (shape->priv->path);
-		/* Render, until SVP */
-		vpath = art_bez_path_to_vec (gnome_canvas_path_def_bpath (cpath), 0.1);
-		gnome_canvas_path_def_unref (cpath);
-
-		svp = art_svp_from_vpath (vpath);
-		art_free (vpath);
-		
-		swr = art_svp_writer_rewind_new (shape->priv->wind);
-		art_svp_intersector (svp, swr);
-		
-		svp2 = art_svp_writer_rewind_reap (swr);
-		art_svp_free (svp);
-  
-		art_drect_svp (&bbox, svp2);
-		art_svp_free (svp2);
-	}
-
-	*x1 = bbox.x0;
-	*y1 = bbox.y0;
-	*x2 = bbox.x1;
-	*y2 = bbox.y1;
-}
